/*********************************************************************
Example written by Limor Fried/Ladyada  for Adafruit Industries.  
BSD license, check license.txt for more information
All text above, and the splash screen must be included in any redistribution
*********************************************************************/


#include <SPIFFS.h>  //add   
#include <HTTPClient.h> //add  
#include <SD.h> //add  
#include <WifiClientSecure.h>
#include <WiFi.h>

#include <SPI.h>
#include <Wire.h>
//#include <Adafruit_GFX.h>
#include <U8g2lib.h>

#include "esp_sleep.h"
#include "nvs.h"
#include "nvs_flash.h"
#include "soc/rtc_cntl_reg.h"
#include "soc/rtc_io_reg.h"
#include "soc/sens_reg.h"
#include "soc/soc.h"
#include "driver/gpio.h"
#include "driver/rtc_io.h"
#include "esp32/ulp.h"
#include "ulp_main.h"
#include "ulptool.h"


// ULP 
extern const uint8_t ulp_main_bin_start[] asm("_binary_ulp_main_bin_start");
extern const uint8_t ulp_main_bin_end[]   asm("_binary_ulp_main_bin_end");
static void init_run_ulp(uint32_t usec);
#define BUTTON_PIN_BITMASK 0x300000000 // Wakeup mask for external interrups

// Time keeping
RTC_DATA_ATTR uint32_t time_hour=0;
RTC_DATA_ATTR uint32_t time_minute=0;
RTC_DATA_ATTR uint32_t time_second=0;
RTC_DATA_ATTR uint8_t get_calendar_time_hour;
RTC_DATA_ATTR uint8_t get_calendar_time_minute;
RTC_DATA_ATTR uint32_t ulp_timer_calibration_value;
RTC_DATA_ATTR uint32_t ulp_time_counter_saved;
RTC_DATA_ATTR uint8_t timout_value;

RTC_DATA_ATTR uint8_t time_get_calendar_state; //(0: do nothing, 1: get calendar from google, 2: got a valid calendar)
unsigned long boot_time;

RTC_DATA_ATTR uint64_t sleep_time;


// Holds the battery voltage
float current_battery_voltage=0;

// Should we get the calender from google?



// 'menu' image 96x68px
const unsigned char menu [] = {
0x3f, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x1f, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x60, 0x00, 0x00, 0x00, 
0x00, 0x0c, 0x30, 0x00, 0x00, 0x00, 0x00, 0x06, 0xc0, 0x00, 0xf0, 0x00, 0x00, 0x06, 0x60, 0x00, 
0x06, 0x00, 0x00, 0x03, 0x80, 0x01, 0xb8, 0x00, 0x00, 0x02, 0x40, 0x00, 0x0f, 0x80, 0x00, 0x01, 
0x80, 0x01, 0xe8, 0x00, 0x00, 0x02, 0x40, 0x0e, 0x18, 0xf0, 0x00, 0x01, 0x80, 0x01, 0xec, 0x00, 
0x00, 0x02, 0x40, 0x1b, 0xb0, 0x3c, 0x00, 0x01, 0x80, 0x00, 0xf4, 0x00, 0x00, 0x02, 0x40, 0x18, 
0xf0, 0x07, 0x00, 0x01, 0x80, 0x00, 0x54, 0x00, 0x00, 0x02, 0x40, 0x10, 0x38, 0x07, 0x00, 0x01, 
0x80, 0x00, 0x7e, 0x00, 0x00, 0x02, 0x40, 0x10, 0x0e, 0x1c, 0x00, 0x01, 0x80, 0x00, 0x26, 0x00, 
0x00, 0x02, 0x40, 0x10, 0x06, 0x18, 0x00, 0x01, 0x80, 0x00, 0x63, 0x00, 0x00, 0x02, 0x40, 0x10, 
0x06, 0x30, 0x1c, 0x01, 0x80, 0x00, 0xd1, 0x80, 0x00, 0x02, 0x40, 0x30, 0x06, 0x3f, 0xf7, 0x01, 
0x80, 0x00, 0x90, 0xc0, 0x00, 0x02, 0x4f, 0xf8, 0x06, 0x27, 0xc1, 0xc1, 0x80, 0x00, 0x94, 0x40, 
0x00, 0x02, 0x4e, 0x18, 0x06, 0x20, 0x00, 0x71, 0x80, 0x01, 0x94, 0x60, 0x00, 0x02, 0x47, 0x0c, 
0x02, 0x20, 0x00, 0x31, 0x80, 0x01, 0x94, 0x20, 0x00, 0x02, 0x41, 0x87, 0x03, 0x30, 0x00, 0xe1, 
0x80, 0x01, 0x9c, 0x20, 0x00, 0x02, 0x4c, 0xe1, 0xe1, 0x98, 0x01, 0xf1, 0x80, 0x01, 0x9e, 0x38, 
0x00, 0x02, 0x46, 0x30, 0x7f, 0xdc, 0x07, 0xf9, 0x87, 0x81, 0x8a, 0x1f, 0xfe, 0x02, 0x43, 0x98, 
0x07, 0xff, 0xff, 0xf9, 0x8d, 0xff, 0x2a, 0x18, 0x03, 0x02, 0x49, 0xce, 0x03, 0xff, 0xf8, 0x01, 
0x8c, 0x19, 0x2b, 0x0f, 0xc3, 0x02, 0x4e, 0x63, 0xff, 0xe0, 0x00, 0x01, 0x86, 0x03, 0x2d, 0x04, 
0x0e, 0x02, 0x43, 0x31, 0x80, 0x00, 0x1f, 0xf9, 0x83, 0x82, 0x2d, 0x83, 0xfc, 0x02, 0x41, 0x9c, 
0x01, 0xff, 0xff, 0x01, 0x80, 0xfe, 0x7e, 0x81, 0xe0, 0x02, 0x4c, 0xe7, 0xff, 0xf0, 0x00, 0x01, 
0x80, 0x0c, 0x52, 0x80, 0x78, 0x02, 0x47, 0x33, 0x00, 0x00, 0x07, 0xf9, 0x83, 0xf8, 0xb3, 0x40, 
0x0f, 0x02, 0x41, 0x98, 0x00, 0x3f, 0xff, 0xc1, 0xbf, 0xc3, 0x21, 0xe0, 0x01, 0xe2, 0x40, 0xcf, 
0xff, 0xfe, 0x00, 0x01, 0xb0, 0x00, 0x40, 0xf0, 0x00, 0x32, 0x40, 0x77, 0xf0, 0x00, 0x00, 0x31, 
0x9f, 0xff, 0x80, 0x3f, 0x00, 0x1a, 0x40, 0x38, 0x00, 0x01, 0xff, 0xf1, 0x87, 0xfe, 0x00, 0x03, 
0xf0, 0x3a, 0x40, 0x0c, 0x1f, 0xff, 0xf0, 0x01, 0xc0, 0x00, 0x00, 0x00, 0x3f, 0xe6, 0x60, 0x07, 
0xff, 0x00, 0x00, 0x03, 0x60, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x30, 0x00, 0x00, 0x00, 0x00, 0x06, 
0x3f, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x1f, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x1f, 0xff, 0xff, 0xff, 0xff, 0xfc, 
0x60, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x30, 0x00, 0x00, 0x00, 0x00, 0x06, 0xc0, 0x00, 0x0f, 0xe0, 
0x00, 0x06, 0x60, 0x00, 0x00, 0x00, 0x00, 0x03, 0x80, 0x00, 0x1c, 0x70, 0x00, 0x02, 0x40, 0x00, 
0xff, 0xff, 0x80, 0x01, 0x80, 0x00, 0x38, 0x38, 0x00, 0x02, 0x40, 0x03, 0x86, 0x00, 0xc0, 0x01, 
0x80, 0x7f, 0xff, 0xff, 0xfc, 0x02, 0x40, 0x02, 0x01, 0x00, 0x60, 0x01, 0x80, 0xff, 0xff, 0xff, 
0xfe, 0x02, 0x40, 0x06, 0x01, 0x80, 0x33, 0x01, 0x81, 0xc0, 0x00, 0x00, 0x03, 0x02, 0x40, 0x0c, 
0x00, 0xc0, 0x1f, 0x01, 0x81, 0x80, 0x00, 0x00, 0x03, 0x02, 0x40, 0x18, 0x01, 0xe0, 0x06, 0x01, 
0x81, 0xc0, 0x00, 0x00, 0x07, 0x02, 0x40, 0x30, 0x03, 0x30, 0x0c, 0x01, 0x80, 0xff, 0xff, 0xff, 
0xfe, 0x02, 0x40, 0x38, 0x06, 0x18, 0x18, 0x01, 0x80, 0x00, 0x00, 0x00, 0x00, 0x02, 0x40, 0x07, 
0x0c, 0x38, 0x10, 0x01, 0x80, 0x00, 0x00, 0x00, 0x00, 0x02, 0x5f, 0xfd, 0xf9, 0xff, 0xe7, 0xc1, 
0x80, 0x30, 0x00, 0x00, 0x38, 0x02, 0x5f, 0xfe, 0x30, 0x00, 0x1c, 0x41, 0x80, 0x38, 0xc3, 0x86, 
0x38, 0x02, 0x43, 0x03, 0x00, 0x00, 0xe0, 0x61, 0x80, 0x38, 0xc3, 0x86, 0x38, 0x02, 0x46, 0x01, 
0x80, 0x01, 0x80, 0x31, 0x80, 0x38, 0xc3, 0x86, 0x38, 0x02, 0x4c, 0x00, 0xc0, 0x00, 0x80, 0x19, 
0x80, 0x38, 0xc3, 0x86, 0x38, 0x02, 0x58, 0x03, 0xe0, 0x00, 0xc0, 0x0d, 0x80, 0x38, 0xc3, 0x86, 
0x38, 0x02, 0x58, 0x06, 0xe0, 0x06, 0x60, 0x0d, 0x80, 0x38, 0xc3, 0x86, 0x38, 0x02, 0x48, 0x04, 
0x10, 0x0e, 0x30, 0x0d, 0x80, 0x38, 0xc3, 0x86, 0x38, 0x02, 0x4c, 0x0c, 0x00, 0x0e, 0x18, 0x19, 
0x80, 0x38, 0xc3, 0x86, 0x38, 0x02, 0x46, 0x1f, 0xfe, 0x1f, 0xff, 0xf1, 0x80, 0x38, 0xc3, 0x86, 
0x38, 0x02, 0x43, 0x30, 0x02, 0x30, 0x00, 0x61, 0x80, 0x38, 0xc3, 0x86, 0x38, 0x02, 0x41, 0x60, 
0x02, 0x60, 0x00, 0xc1, 0x80, 0x38, 0xc3, 0x86, 0x38, 0x02, 0x40, 0xc0, 0x02, 0x40, 0x01, 0x81, 
0x80, 0x38, 0xc3, 0x86, 0x38, 0x02, 0x40, 0xc0, 0x02, 0x60, 0x03, 0x01, 0x80, 0x38, 0xc3, 0x86, 
0x38, 0x02, 0x40, 0x60, 0x02, 0x20, 0x06, 0x01, 0x80, 0x38, 0xc1, 0x06, 0x38, 0x02, 0x40, 0x30, 
0x02, 0x30, 0x1c, 0x01, 0x80, 0x18, 0x00, 0x00, 0x30, 0x02, 0x40, 0x0f, 0xfe, 0x1f, 0x70, 0x01, 
0x80, 0x1f, 0x00, 0x01, 0xf0, 0x02, 0x40, 0x00, 0x00, 0x0e, 0x00, 0x01, 0xc0, 0x07, 0xff, 0xff, 
0xc0, 0x06, 0x60, 0x00, 0x00, 0x06, 0x00, 0x03, 0x60, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x30, 0x00, 
0x00, 0x00, 0x00, 0x06, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x1f, 0xff, 0xff, 0xff, 0xff, 0xfc};

const unsigned char menui [] = {// 'menu', 96x68px
// 'menu', 96x68px
0x3f, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x1f, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x7f, 0xff, 0xff, 0xff, 
0xff, 0xfc, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0x0f, 0xff, 0xff, 0xfe, 0x7f, 0xff, 
0xf9, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x47, 0xff, 0xff, 0xfe, 0x7f, 0xff, 0xf0, 0x7f, 0xff, 0xff, 
0xff, 0xfe, 0x17, 0xff, 0xff, 0xfe, 0x7f, 0xf1, 0xe7, 0x0f, 0xff, 0xff, 0xff, 0xfe, 0x13, 0xff, 
0xff, 0xfe, 0x7f, 0xe4, 0x4f, 0xc3, 0xff, 0xff, 0xff, 0xff, 0x0b, 0xff, 0xff, 0xfe, 0x7f, 0xe7, 
0x0f, 0xf8, 0xff, 0xff, 0xff, 0xff, 0xab, 0xff, 0xff, 0xfe, 0x7f, 0xef, 0xc7, 0xf8, 0xff, 0xff, 
0xff, 0xff, 0x81, 0xff, 0xff, 0xfe, 0x7f, 0xef, 0xf1, 0xe3, 0xff, 0xff, 0xff, 0xff, 0xd9, 0xff, 
0xff, 0xfe, 0x7f, 0xef, 0xf9, 0xe7, 0xff, 0xff, 0xff, 0xff, 0x9c, 0xff, 0xff, 0xfe, 0x7f, 0xef, 
0xf9, 0xcf, 0xe3, 0xff, 0xff, 0xff, 0x2e, 0x7f, 0xff, 0xfe, 0x7f, 0xcf, 0xf9, 0xc0, 0x08, 0xff, 
0xff, 0xff, 0x6f, 0x3f, 0xff, 0xfe, 0x70, 0x07, 0xf9, 0xd8, 0x3e, 0x3f, 0xff, 0xff, 0x6b, 0xbf, 
0xff, 0xfe, 0x71, 0xe7, 0xf9, 0xdf, 0xff, 0x8f, 0xff, 0xfe, 0x6b, 0x9f, 0xff, 0xfe, 0x78, 0xf3, 
0xfd, 0xdf, 0xff, 0xcf, 0xff, 0xfe, 0x6b, 0xdf, 0xff, 0xfe, 0x7e, 0x78, 0xfc, 0xcf, 0xff, 0x1f, 
0xff, 0xfe, 0x63, 0xdf, 0xff, 0xfe, 0x73, 0x1e, 0x1e, 0x67, 0xfe, 0x0f, 0xff, 0xfe, 0x61, 0xc7, 
0xff, 0xfe, 0x79, 0xcf, 0x80, 0x23, 0xf8, 0x07, 0xf8, 0x7e, 0x75, 0xe0, 0x01, 0xfe, 0x7c, 0x67, 
0xf8, 0x00, 0x00, 0x07, 0xf2, 0x00, 0xd5, 0xe7, 0xfc, 0xfe, 0x76, 0x31, 0xfc, 0x00, 0x07, 0xff, 
0xf3, 0xe6, 0xd4, 0xf0, 0x3c, 0xfe, 0x71, 0x9c, 0x00, 0x1f, 0xff, 0xff, 0xf9, 0xfc, 0xd2, 0xfb, 
0xf1, 0xfe, 0x7c, 0xce, 0x7f, 0xff, 0xe0, 0x07, 0xfc, 0x7d, 0xd2, 0x7c, 0x03, 0xfe, 0x7e, 0x63, 
0xfe, 0x00, 0x00, 0xff, 0xff, 0x01, 0x81, 0x7e, 0x1f, 0xfe, 0x73, 0x18, 0x00, 0x0f, 0xff, 0xff, 
0xff, 0xf3, 0xad, 0x7f, 0x87, 0xfe, 0x78, 0xcc, 0xff, 0xff, 0xf8, 0x07, 0xfc, 0x07, 0x4c, 0xbf, 
0xf0, 0xfe, 0x7e, 0x67, 0xff, 0xc0, 0x00, 0x3f, 0xc0, 0x3c, 0xde, 0x1f, 0xfe, 0x1e, 0x7f, 0x30, 
0x00, 0x01, 0xff, 0xff, 0xcf, 0xff, 0xbf, 0x0f, 0xff, 0xce, 0x7f, 0x88, 0x0f, 0xff, 0xff, 0xcf, 
0xe0, 0x00, 0x7f, 0xc0, 0xff, 0xe6, 0x7f, 0xc7, 0xff, 0xfe, 0x00, 0x0f, 0xf8, 0x01, 0xff, 0xfc, 
0x0f, 0xc6, 0x7f, 0xf3, 0xe0, 0x00, 0x0f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xc0, 0x1e, 0x7f, 0xf8, 
0x00, 0xff, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xfe, 
0x3f, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x1f, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
0x00, 0x00, 0x00, 0x00, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x1f, 0xff, 0xff, 0xff, 0xff, 0xfc, 
0x7f, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xff, 0xff, 0xf0, 0x1f, 
0xff, 0xfe, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xe3, 0x8f, 0xff, 0xfe, 0x7f, 0xff, 
0x00, 0x00, 0x7f, 0xff, 0xff, 0xff, 0xc7, 0xc7, 0xff, 0xfe, 0x7f, 0xfc, 0x79, 0xff, 0x3f, 0xff, 
0xff, 0x80, 0x00, 0x00, 0x03, 0xfe, 0x7f, 0xfd, 0xfe, 0xff, 0x9f, 0xff, 0xff, 0x00, 0x00, 0x00, 
0x01, 0xfe, 0x7f, 0xf9, 0xfe, 0x7f, 0xcc, 0xff, 0xfe, 0x3f, 0xff, 0xff, 0xfc, 0xfe, 0x7f, 0xf3, 
0xff, 0x3f, 0xe0, 0xff, 0xfe, 0x7f, 0xff, 0xff, 0xfc, 0xfe, 0x7f, 0xe7, 0xfe, 0x1f, 0xf9, 0xff, 
0xfe, 0x3f, 0xff, 0xff, 0xf8, 0xfe, 0x7f, 0xcf, 0xfc, 0xcf, 0xf3, 0xff, 0xff, 0x00, 0x00, 0x00, 
0x01, 0xfe, 0x7f, 0xc7, 0xf9, 0xe7, 0xe7, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x7f, 0xf8, 
0xf3, 0xc7, 0xef, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0x60, 0x02, 0x06, 0x00, 0x18, 0x3f, 
0xff, 0xcf, 0xff, 0xff, 0xc7, 0xfe, 0x60, 0x01, 0xcf, 0xff, 0xe3, 0xbf, 0xff, 0xc7, 0x3c, 0x79, 
0xc7, 0xfe, 0x7c, 0xfc, 0xff, 0xff, 0x1f, 0x9f, 0xff, 0xc7, 0x3c, 0x79, 0xc7, 0xfe, 0x79, 0xfe, 
0x7f, 0xfe, 0x7f, 0xcf, 0xff, 0xc7, 0x3c, 0x79, 0xc7, 0xfe, 0x73, 0xff, 0x3f, 0xff, 0x7f, 0xe7, 
0xff, 0xc7, 0x3c, 0x79, 0xc7, 0xfe, 0x67, 0xfc, 0x1f, 0xff, 0x3f, 0xf3, 0xff, 0xc7, 0x3c, 0x79, 
0xc7, 0xfe, 0x67, 0xf9, 0x1f, 0xf9, 0x9f, 0xf3, 0xff, 0xc7, 0x3c, 0x79, 0xc7, 0xfe, 0x77, 0xfb, 
0xef, 0xf1, 0xcf, 0xf3, 0xff, 0xc7, 0x3c, 0x79, 0xc7, 0xfe, 0x73, 0xf3, 0xff, 0xf1, 0xe7, 0xe7, 
0xff, 0xc7, 0x3c, 0x79, 0xc7, 0xfe, 0x79, 0xe0, 0x01, 0xe0, 0x00, 0x0f, 0xff, 0xc7, 0x3c, 0x79, 
0xc7, 0xfe, 0x7c, 0xcf, 0xfd, 0xcf, 0xff, 0x9f, 0xff, 0xc7, 0x3c, 0x79, 0xc7, 0xfe, 0x7e, 0x9f, 
0xfd, 0x9f, 0xff, 0x3f, 0xff, 0xc7, 0x3c, 0x79, 0xc7, 0xfe, 0x7f, 0x3f, 0xfd, 0xbf, 0xfe, 0x7f, 
0xff, 0xc7, 0x3c, 0x79, 0xc7, 0xfe, 0x7f, 0x3f, 0xfd, 0x9f, 0xfc, 0xff, 0xff, 0xc7, 0x3c, 0x79, 
0xc7, 0xfe, 0x7f, 0x9f, 0xfd, 0xdf, 0xf9, 0xff, 0xff, 0xc7, 0x3e, 0xf9, 0xc7, 0xfe, 0x7f, 0xcf, 
0xfd, 0xcf, 0xe3, 0xff, 0xff, 0xe7, 0xff, 0xff, 0xcf, 0xfe, 0x7f, 0xf0, 0x01, 0xe0, 0x8f, 0xff, 
0xff, 0xe0, 0xff, 0xfe, 0x0f, 0xfe, 0x7f, 0xff, 0xff, 0xf1, 0xff, 0xff, 0xff, 0xf8, 0x00, 0x00, 
0x3f, 0xfe, 0x7f, 0xff, 0xff, 0xf9, 0xff, 0xff, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xfc, 0x3f, 0xff, 
0xff, 0xff, 0xff, 0xfe, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xf8, 0x1f, 0xff, 0xff, 0xff, 0xff, 0xfc

};

// Possible bin events and menu handling
#define NBR_EVENTS 4
String  event_names[NBR_EVENTS] = {"RubbishBin", "GreenBin",  "PaperBin", "YellowBag"};
RTC_DATA_ATTR uint8_t event_state[NBR_EVENTS]={0};
uint8_t menu_item=4;
bool blink_show=false;

// Timing stuff

hw_timer_t * timer = NULL;
int timeout_counter = 0;

int debug=0;

#ifndef _BV
#define _BV(x) (1 << (x))
#endif

// LCD pins
#define lcd_RST_pin 22
#define lcd_CS_pin 19
#define lcd_DIN_pin 23
#define lcd_CLK_pin 18
#define lcd_BL_pin 13 //(move to GPIO13 - RTC_GPIO14)
#define but_1_pin 33
#define but_2_pin 32
#define ADC_pin 35

RTC_DATA_ATTR int bootCount = 0;
RTC_DATA_ATTR uint8_t last_hour=50;
RTC_DATA_ATTR uint32_t last_ulp_lcd_mode; //Saves the backlight mode


#define WHITE 0xFF
#define BLACK 0xFF

U8G2_HX1230_96X68_F_3W_SW_SPI display(U8G2_R2,lcd_CLK_pin, lcd_DIN_pin, lcd_CS_pin, lcd_RST_pin);
//U8G2_HX1230_96X68_F_3W_HW_SPI display(U8G2_R2, lcd_CS_pin, lcd_RST_pin);


//Network credentials
const char*  myssid = "yourssid";
const char* mypassword = "yourpassword"; //replace with your password

//Google calender credentials
const char*  server = "script.google.com";  // Server URL
String URL="";
// google script key
const char* key = "yourkey";


// Battery history

#define battery_history_length 90
bool show_battery_graph=false;
RTC_DATA_ATTR uint16_t battery_history [battery_history_length+10]={377};


// This adds a new datapoint to the battery history
void update_battery_history( bool iterate, bool randomize)
{
  for (int this_time=0; this_time < battery_history_length-1; this_time++)
  {
  if (iterate)
     battery_history[this_time]=battery_history[this_time+1];    
  if (randomize)
     battery_history[this_time]=int(random(0,400));
  
  }
  battery_history[ battery_history_length-1]=int(current_battery_voltage*100.0);

}

// Draws a graph of the battery history
void draw_battery_graph()
{
  uint8_t x_off=4;
  uint8_t y_off=48;
  //Drawing Axis
  display.clearBuffer();
  display.setDrawColor(1);
   display.drawFrame(0,0,95,66);
  display.drawLine(0,48,94,48);
  display.drawLine(x_off+90,y_off,x_off+90,0);
  for (int this_line=0; this_line <5; this_line++)
  {
     display.drawLine(0,48-30*this_line,3,48-30*this_line);
    display.drawLine(91,48-30*this_line,94,48-30*this_line);

      display.drawLine(94-this_line*24,y_off,94-this_line*24,y_off-3);
      display.drawLine(94-this_line*24,0,94-this_line*24,3);
   
  }


  //Drawing Graph
  for (int this_time=0; this_time < battery_history_length; this_time++)
    display.drawPixel(this_time+x_off,y_off-int(battery_history[this_time]/10));
  float battery_voltage= battery_history[battery_history_length-1]/100.0;
  if (battery_voltage>4.1)
    battery_voltage=4.1;
  if (battery_voltage<3.1)
    battery_voltage=3.1;
  uint8_t battery_indicator = int ((battery_voltage- 3.1)*22.0);
  
  //Drawing Battery
  // Battery ourline
  display.drawFrame(x_off+4,y_off+2,25,14);
  
  display.drawBox(x_off,y_off+6,4,6);
  // Battery indicator
  display.drawBox(x_off+2+25-battery_indicator,y_off+4,battery_indicator,10);

  //Drawing Numerals"
  //display.setCursor(x_off+22,y_off+2);
  //display.setTextColor(1);
  //display.setTextSize(2);
  display.setFont(u8g2_font_haxrcorp4089_t_cyrillic          );  // choose a suitable font
  char time_char[8];
  sprintf(time_char, "%02d:%02d", time_hour, time_minute);
  String display_string = String(battery_history[battery_history_length-1]/100.0) + "V" + " | " + String(time_char);
  
  display.drawStr(x_off+31,y_off+13,display_string.c_str());  // write something to the internal memory

  display.sendBuffer();
 
}

// Draws the menu icons
void draw_icon (uint8_t icon, bool selected, bool draw_enable)
{
  uint8_t icon_width=48;
  uint8_t icon_height=34;
  uint8_t x_start=0;
  uint8_t y_start=0;

//  {"RubbishBin", "GreenBin",  "PaperBin", "YellowBag"};
  
  if (icon==0)
  {
    x_start=0;
    y_start=34;
  }
  if (icon==1)
  {
    x_start=0;
    y_start=0;
  }
  if (icon==2)
  {
    x_start=48;
    y_start=0;
  }
  if (icon==3)
  {
    x_start=48;
    y_start=34;
  }
   
  display.setDrawColor(0);
  display.drawBox(x_start,y_start,icon_width,icon_height);
  
  display.setDrawColor(draw_enable);
      
  for (int yy=y_start; yy<(y_start+icon_height);yy++)
  {
    for (int xx=x_start; xx<(x_start+icon_width); xx++)
    {
      if ((selected) && (menui[yy*12+xx / 8] & _BV(7-xx%8)))
        display.drawPixel(xx,yy);
      if ((!selected) && (menu[yy*12+xx / 8] & _BV(7-xx%8)))
        display.drawPixel(xx,yy);
    }
   
  }
 
   display.sendBuffer();
}



    
// STart ULP processor
void start_ulp(bool calibrate)
{

  vTaskDelay( 10 / portTICK_PERIOD_MS );
  
  // Configure & start ULP for LCD backlight (we wil control pin 13 from ULP code)
  rtc_gpio_init(GPIO_NUM_13);
  rtc_gpio_set_direction(GPIO_NUM_13, RTC_GPIO_MODE_OUTPUT_ONLY);
  rtc_gpio_set_level(GPIO_NUM_13,1);
  ESP_ERROR_CHECK( ulp_load_binary(0, ulp_main_bin_start, (ulp_main_bin_end - ulp_main_bin_start) / sizeof(uint32_t)));
  ESP_ERROR_CHECK( ulp_run((&ulp_entry - RTC_SLOW_MEM) / sizeof(uint32_t)));
  
  
  if (calibrate)
  {
    unsigned long delta_time;
    unsigned long start_time;
    double calibration_factor;
    timeout_counter=0;
    Serial.println("Calibrating ULP timer");
    ulp_timer_calibration_value=4000;
    ulp_timer_calibrate=ulp_timer_calibration_value;
    Serial.println("Old ULP calibration value (ulp_timer_calibrate): " + String(ulp_timer_calibration_value));
   
   
    
    while ((ulp_timer_calibration_value<4500) || (ulp_timer_calibration_value>6000))
    {
    
    
      Serial.println("Start calibration...");
 
      
      yield();
      ulp_time_counter=0;
      
      while ((ulp_time_counter & 0xFFFF)==0)
        delayMicroseconds(1);

      
      start_time=micros();
      
      while ((ulp_time_counter & 0xFFFF)==1)
        delayMicroseconds(1);
      delta_time=micros()-start_time;
    
   
      calibration_factor = 1000000.0 / (double)delta_time;
      ulp_timer_calibration_value = int((double)ulp_timer_calibration_value*calibration_factor)+10;
      Serial.println("ULP calibration value (ulp_timer_calibrate): " + String(ulp_timer_calibration_value));
      
      yield();
    }
  
    
    Serial.println("One second duration in us: "+ String(delta_time));
    Serial.println("New ULP calibration value (ulp_timer_calibrate): " + String(ulp_timer_calibration_value));
    timeout_counter=0;
      yield();
  }
  ulp_timer_calibrate = ulp_timer_calibration_value; 
  

}

// Puts the ESP in deep sleep
void goToSleep(){

    get_ulp_time(0); // Update current time from ULP
   
    start_ulp(0);
    
    if (time_get_calendar_state==1)
      sleep_time = 1000000ULL *60ULL *5ULL ; // Retry in 5 minutes
    else
      sleep_time = 1000000ULL *60ULL * (unsigned long long int)(62-time_minute) ;  // Wake up 2 minutes after the hour
    
    
    // Stop blink timer
    timerAlarmDisable(timer);
    // Show plain menu
    menu_item=4;
    
    
    
    ulp_lcd_mode=0;
    for (int i=0; i< NBR_EVENTS; i++)
    {
        if (event_state[i]==1)
        {
          draw_icon(i,1,1);
        }
        else
          draw_icon(i,0,1);

        if ((event_state[i]==1) && (time_hour>19))
          ulp_lcd_mode=2; 
          
    } 
    // Disable chip select for display
    digitalWrite(lcd_CS_pin, HIGH);
    // Hold chip select high during sleep
    gpio_hold_en(GPIO_NUM_19);
    gpio_deep_sleep_hold_en();

    // Sleep 60 secs and define wakeup sources
    
    esp_sleep_enable_timer_wakeup(sleep_time); 
    esp_sleep_enable_ext1_wakeup(BUTTON_PIN_BITMASK,ESP_EXT1_WAKEUP_ANY_HIGH);
    //esp_sleep_enable_ext0_wakeup(GPIO_NUM_16,1);
    esp_deep_sleep_start();
}

// Blinking timer (0.5 Hz)
void IRAM_ATTR onTimer(){
  timeout_counter++;

  if (time_get_calendar_state==1)
    timout_value=120; // 1 minute timout if we get a calender
  else
    timout_value=20; // 10 secs timout otherwise
  
  
  if (timeout_counter>timout_value)
    goToSleep();
  
  blink_show=!blink_show;
  if (show_battery_graph==1)
  {
   update_battery_history(0,0);
   draw_battery_graph(); 
   //show_battery_graph=2;
  }

  if (show_battery_graph==0)
  {
    for (int i=0; i< NBR_EVENTS; i++)
    {
      if (i==menu_item)
        draw_icon(i,event_state[i]==1,blink_show);
      else
        draw_icon(i,event_state[i]==1,1);
    }
  }
}

//Connect to wifi
void connectToWifi() {
  Serial.println();
  Serial.print("Connecting to wifi: ");
  Serial.println(myssid);

  WiFi.begin(myssid, mypassword);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("WiFi connected ");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());

}


// Get calendar and current time from google
bool getCalendar() {

  vTaskDelay( 10 / portTICK_PERIOD_MS );
  WiFiClientSecure client, client2;

  String movedURL;
  String line;
  client.setInsecure();
  client.setTimeout(3000);
  if (debug)Serial.println("Verbinde zum script.google.com");
  if (!client.connect(server, 443))
  {
    if (debug) Serial.println("Verbindung fehlgeschlagen!");
    return false;
  }

  client.println("GET " + URL + " HTTP/1.1");
  if (debug) Serial.println("Requesting: GET " + URL + " HTTP/1.1");
  client.println("Host: script.google.com" );
  client.println("Connection: close"); 
  client.println();

   // ESP32 empfängt antwort vom Google sheets
  while (client.connected())     // ESP32  empfängt Header
  {
    line = client.readStringUntil('\n');
    if (debug) Serial.println(line);
    if (line == "\r") break;      // Ende Des Headers empfangen
    if (line.indexOf ( "Location" ) >= 0)   // Weiterleitung im Header?
    { // Neue URL merken
      movedURL = line.substring ( line.indexOf ( ":" ) + 2 ) ;
    }
  }
  yield();
  while (client.connected())    // Google Antwort HTML Zeilenweise Lesen
  {
    if (client.available())
    {
      line = client.readStringUntil('\r');
      if (debug) Serial.print(line);
    }
  }
  client.stop();
  yield();
 
  movedURL.trim(); // leerzeichen, \n entfernen
  if (debug) Serial.println("Weiterleitungs URL: \"" + movedURL + "\"");
 
  if (movedURL.length() < 10) return false; // Weiterleitung nicht da
 
 client2.setInsecure();
 client2.setTimeout(3000);
  if (debug) Serial.println("\n Starte Weiterleitung...");
  if (!client2.connect(server, 443))
  {
    if (debug) Serial.println("Weiterleitung fehlgeschlagen!");
    return false;
  }
  yield();
  if (debug) Serial.println("Connected!");
  // // ESP32 Erzeugt HTTPS Anfrage an Google Tabellen
  client2.println("GET " + movedURL);
  client2.println("Host: script.google.com");
  client2.println("Connection: close");
  client2.println();
 
  while (client2.connected()) // ESP32  empfängt Header
  {
    line = client2.readStringUntil('\n');
    if (debug) Serial.println(line);
    if (line == "\r")break;
  }
  yield();

  
  while (client2.connected()) // Google Antwort HTML Zeilenweise Lesen
  {
    if (client2.available())
    {
      line = client2.readStringUntil('\r');
      if (debug) Serial.print(line);
    }
  }
  yield();
  client2.stop(); 
  Serial.println(line);

  // No valid data
  if (line.length()<4)
  {
    time_get_calendar_state=1;
    return false;

  }
  // Got valid data
 time_get_calendar_state=2;

 
  
  // We got valid data
  time_hour=line.substring(0,2).toInt();
  time_minute=line.substring(3,5).toInt();

  get_calendar_time_hour=time_hour;
  get_calendar_time_minute=time_minute;
  
  // Update event matrix
  for (int i=0; i< NBR_EVENTS; i++)
  {
    if ((line.indexOf(event_names[i])>-1) && event_state[i]==0)
    {
        event_state[i]=1;
        
    }
    if (line.indexOf(event_names[i])<0)
        event_state[i]=0;
  }
  
  

  return 1;
}

// Delete task button event 
void delete_task()
{
  if (event_state[menu_item]==1)
    event_state[menu_item]=2; 
  menu_item=4;
  if (digitalRead(but_2_pin))
    show_battery_graph=!show_battery_graph;
  
  timeout_counter=0;
}

// Next task button event
void next_task()
{

  menu_item++;
  if (menu_item>3)
    menu_item=0;
  if (digitalRead(but_1_pin))
    show_battery_graph=!show_battery_graph;
  timeout_counter=0;
}


// Sample battery voltage
float get_battery_voltage()
{
  int ADC_VALUE = analogRead(ADC_pin);
  float voltage_value = (ADC_VALUE * 2.0 * 3.3 ) / (4095);
  Serial.println("Battery voltage: " + String(voltage_value));
  return voltage_value;
}

// Compute time from ulp_time_counter (keeps track of time during sleep)
void get_ulp_time(bool boot_up)
{
  //uint32_t time_minutes = (ulp_time_counter-29360131UL)/60;
  unsigned long uptime;
  do {
    ulp_time_counter_saved=ulp_time_counter & 0xFFFF;
  } while(ulp_time_counter_saved>4000);
 
  if (boot_up)
  {
    Serial.println("Time at beginning of sleep: " + String(time_hour)+":"+String(time_minute)+":"+String(time_second));
    Serial.println("Sleep duration in seconds: " + String(ulp_time_counter_saved));
    Serial.println("Get calender state before sleep: " + String(time_get_calendar_state));
    Serial.println("Timout value before sleep: " + String(timout_value));
    
    
    Serial.println("Connection to google made at: " + String(get_calendar_time_hour)+":" + String(get_calendar_time_minute));
  }
  
  if (boot_up)
    time_second += ulp_time_counter_saved; //duration since going to sleep
  else
  {
    uptime=millis() - boot_time;
    if (uptime>0)
      time_second += uptime/1000; //uptime   
  }
    
  
  time_minute += (time_second/60);
  time_hour+= (time_minute/60); 
  time_second=time_second % 60;
  time_minute=time_minute % 60;
  if (time_hour>=24)
    time_hour=0;
    
  ulp_time_counter=0;
  if (boot_up)
  {
   Serial.println("Time now: " + String(time_hour)+":"+String(time_minute)+":"+String(time_second));
   Serial.println("ULP calibration value (ulp_timer_calibrate): " + String(ulp_timer_calibration_value));
   Serial.println("Last ulp_time_counter before sleep: " + String(ulp_time_counter_saved));
  }
}

void print_wakeup_reason(){
  esp_sleep_wakeup_cause_t wakeup_reason;

  wakeup_reason = esp_sleep_get_wakeup_cause();

  switch(wakeup_reason)
  {
    case ESP_SLEEP_WAKEUP_EXT0 : Serial.println("Wakeup caused by external signal using RTC_IO");   get_ulp_time(1); ulp_lcd_mode=1;break;
    case ESP_SLEEP_WAKEUP_EXT1 : Serial.println("Wakeup caused by external signal using RTC_CNTL");   get_ulp_time(1); ulp_lcd_mode=1;break;
    case ESP_SLEEP_WAKEUP_TIMER : 
      Serial.println("Wakeup caused by timer"); 
      get_ulp_time(1);
      
       // Update battery history graph and sync time / bin itmes at 3am every morning
       if ((time_hour>3)&&(time_get_calendar_state==0))
       {
         update_battery_history(1,0);
         time_get_calendar_state=1;
       }

       // Reset state
       if ((time_hour<7) && (time_get_calendar_state==2))
         time_get_calendar_state=0; 
        
      break;
    case ESP_SLEEP_WAKEUP_TOUCHPAD : Serial.println("Wakeup caused by touchpad"); break;
    case ESP_SLEEP_WAKEUP_ULP : Serial.println("Wakeup caused by ULP program"); break;
    default : Serial.printf("Wakeup was not caused by deep sleep: %d\n",wakeup_reason);   start_ulp(1); time_get_calendar_state=1; get_calendar_time_minute=0; get_calendar_time_hour=0; update_battery_history(0,1); break;
  }
}



void setup()   {  
  Serial.begin(9600);
  print_wakeup_reason();
  boot_time=millis();
  
  gpio_hold_dis(GPIO_NUM_19);
  analogSetPinAttenuation(ADC_pin, ADC_11db);
  pinMode(ADC_pin,INPUT);
  current_battery_voltage = get_battery_voltage();

  // Start display
  display.begin();
  // Start blink timer
  onTimer();              

  // Print current events
  for (int i=0; i< NBR_EVENTS; i++)
    Serial.println(String(event_names[i]) + " " + String(event_state[i]));

  // Build script URL
  URL="https://script.google.com/macros/s/";
  //String URL="/macros/s/";
  URL += key;
  URL += "/exec";

  // Button pins as input  
  pinMode(but_1_pin,INPUT);
  pinMode(but_2_pin,INPUT);

 

  //print_wakeup_reason();

  // Button interrups
  attachInterrupt(digitalPinToInterrupt(but_1_pin), delete_task, RISING);
  attachInterrupt(digitalPinToInterrupt(but_2_pin), next_task, RISING);


  // Start blink timer
   /* Use 1st timer of 4 */
  /* 1 tick take 1/(80MHZ/80) = 1us so we set divider 80 and count up */
  timer = timerBegin(0, 80, true);

  /* Attach onTimer function to our timer */
  timerAttachInterrupt(timer, &onTimer, true);

  /* Set alarm to call onTimer function every second 1 tick is 1us
  => 1 second is 1000000us */
  /* Repeat the alarm (third parameter) */
  timerAlarmWrite(timer, 500000, true);

  /* Start an alarm */
  timerAlarmEnable(timer);
  
}


void loop() {

  if (time_get_calendar_state==1)
  {
    connectToWifi();
    yield();
    getCalendar();
    
  }
    

  yield();
}
